@@ -2,7 +2,7 @@ from __future__ import unicode_literals
 import warnings
 
 from django.core.exceptions import ValidationError
-from django.utils.translation import ugettext as _
+from django.utils.translation import ugettext as _, ngettext
 from django_password_validators.settings import get_password_hasher
 from django_password_validators.password_history.models import (
     PasswordHistory,
@@ -17,6 +17,15 @@ class UniquePasswordsValidator(object):
     The password is only checked for an existing user.
     """
 
+    def __init__(self, last_passwords=0):
+        """
+
+        :param last_passwords:
+            * lookup_range > 0 - We check only the XXX latest passwords
+            * lookup_range <= 0 - Check all passwords that have been used so far
+        """
+        self.last_passwords = int(last_passwords)
+
     def _user_ok(self, user):
         if not user:
             return
@@ -38,11 +47,26 @@ class UniquePasswordsValidator(object):
 
         return True
 
+    def delete_old_passwords(self, user):
+        if self.last_passwords > 0:
+            # Delete old passwords that are outside the lookup_range
+            password_ids = list(
+                PasswordHistory.objects. \
+                    filter(user_config__user=user). \
+                    order_by('-date')[self.last_passwords:]. \
+                    values_list('pk', flat=True)
+            )
+            if password_ids:
+                PasswordHistory.objects.filter(pk__in=password_ids).delete()
+
     def validate(self, password, user=None):
 
         if not self._user_ok(user):
             return
 
+        # We make sure there are no old passwords in the database.
+        self.delete_old_passwords(user)
+
         for user_config in UserPasswordHistoryConfig.objects.filter(user=user):
             password_hash = user_config.make_password_hash(password)
             try:
@@ -86,6 +110,16 @@ class UniquePasswordsValidator(object):
             ols_password.password = password_hash
             ols_password.save()
 
+        # We make sure there are no old passwords in the database.
+        self.delete_old_passwords(user)
+
     def get_help_text(self):
-        return _('Your new password can not be identical to any of the '
-                 'previously entered.')
+        if self.last_passwords > 0:
+            return ngettext(
+                'Your new password can not be identical to any of the %(old_pass)d previously entered passwords.',
+                'Your new password can not be identical to any of the %(old_pass)d previously entered passwords.',
+                self.last_passwords
+            ) % {'old_pass': self.last_passwords}
+
+        else:
+            return _('Your new password can not be identical to any of the previously entered.')
